#ifndef __FASTFIT_H__
#define __FASTFIT_H__

#include "MathGold.h"
#include "TransformResiduals.h"
#include "MathCholesky.h"
#include "TraitTransformations.h"
#include "MathConstant.h"
#include "MathSVD.h"

//the same definition can be found in MathGold.cpp
#define GLIMIT             100      // Maximum magnification, per round
#define SHIFT(a, b, c, d)  (a)=(b); (b)=(c); (c)=(d)
#define SHFT3(a, b, c)     (a)=(b); (b)=(c)


//This class uses optimization method proposed in FaST paper by Lippert 
//from Nature Vol.8 No.10 833-835 2011
class FastFit : public ScalarMinimizer
{
	public:
		FastFit() {};
		~FastFit() {};
		virtual double Brent(double tol, FastTransform & trans);
		static bool   useCovariates;
		static bool   inverseNormal;
		static bool   regressOutCov;
		static bool   unrelated;
		static String traitName;

		int GetTraitID(Pedigree & ped, const char * name);

		//Estimated parameters are saved here
		double sigma_g2Hat,sigma_e2Hat,deltaHat,sigma_gXHat,sigma_s2Hat,logLikelihood,sigma2Hat;
		Vector beta;
		Vector betaHat;
Vector residuals_unrelated;

		//Estimation functions
		double Evaluate(double delta,FastTransform & trans,bool full);
		double EvaluateMulti(FastTransform & trans,bool full);
		void GetBeta(double delta, FastTransform & trans);
		double GetSigma_g(double delta,Vector & beta, FastTransform & trans);

		//This is where data is transformed and parameters are estimated 
		void FitLinearModels(AutoFit & engine,Pedigree & ped, double tol, FastTransform & trans,String & traitName);

		void EstimateAnalyticalBeta(Vector & Y,FastTransform & trans,Vector & beta_hat);

		//void EstimateAnalyticalSigma2(FastTransform & trans,int trait);

		//optimization
		void OptimizeDelta(double tol,FastTransform & trans);

		//void FitPolygenicModels(Pedigree & ped, bool quiet = false);
};

#endif


