////////////////////////////////////////////////////////////////////// 
// TransformResiduals.h 
// (c) 2012-2013 Shuang Feng, Dajiang Liu, Goncalo Abecasis
// 
// This file is distributed as part of the RareMetalWorker source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile RareMetalWorker.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Wednesday November 28, 2012
// 

#ifndef __TRANSFORMRESIDULAS_H__
#define __TRANSFORMRESIDULAS_H__
 
#include "Kinship.h"
#include "KinshipX.h"
#include "KinshipEmp.h"
#include "MathNormal.h"
#include "MathVector.h"
#include "StringBasics.h"
#include "MathMatrix.h"
#include "MathCholesky.h"
#include "AutoFit.h"

class FastTransform
{
        public:
	FastTransform();
        ~FastTransform();

	static bool empKin;
	static bool pedKin;
	static String readInEmp;
	
	//# of useful families and individuals
	int families,persons,fixedEffects,analyzedFounders,numFounder,totalN,numFamily;
	double traitVar,traitMean;

	//Hash of sample ids from vcf file, if reading from vcf is needed
	StringIntHash sampleVCFIDHash,samplePEDIDHash,foundersHash;

	IntArray * pPheno;
	StringArray founders;
	//Saved Matrices for likelihood calculation
	Matrix X,UX,transU;
	Vector D,UY,Y; 
	Matrix UDX,UD,inv; //here D is D^1/2
	Vector UDY; //D is D^1/2

	//Reformat LL* into UDU*U
	void ReformCholesky(Matrix & L, Vector & D);
	int GetTraitID(Pedigree & ped, const char * name);
	//These function are transformation work horses
	void TransformPedkin(Pedigree & ped, int traitNum,bool useCovaraites);
	void TransformEmpkin(Pedigree & ped, int traitNum,bool useCovariates);
	void GetFixedEffectsCount(Pedigree & ped, bool useCovariates);
	void Prepare(Pedigree & ped, int traitNum,bool useCovariates);
	void TransformPedkinSibEnv(AutoFit & engine, Pedigree &ped);
	void FinalizeTransform(double sigma_g2,double sigma_e2);
	void SubSet(Matrix & kin,Pedigree & ped);
	void SelectSamplesPED(Pedigree & ped);
	void SelectSamplesVCF();
};

#endif
